@@ -1,9 +1,15 @@
-//! Ethernet II protocol per IEEE 802.3
+//! Link layer: Ethernet II protocol per IEEE 802.3
+//! 
 //! Diagram at https://en.wikipedia.org/wiki/Ethernet_frame#Ethernet_II
 
 use crate::{udp::UDPPacket, MACAddr};
 
-/// Combined preamble and start-frame delimiter because they are never changed or separated
+#[cfg(feature = "crc")]
+use crc32fast;
+
+/// Combined 7-byte preamble and 1-byte start-frame delimiter because they are never changed or separated
+/// 
+/// These are usually supplied by the hardware
 const PREAMBLE: [u8; 8] = [
     0b1010_1010,
     0b1010_1010,
@@ -16,6 +22,8 @@ const PREAMBLE: [u8; 8] = [
 ];
 
 /// Standard 96-bit inter-packet gap
+/// 
+/// This is usually supplied by the hardware
 const IPG: [u8; 12] = [0; 12];
 
 /// Header for Ethernet II frame like
@@ -68,7 +76,7 @@ impl EthernetHeader {
     /// Set destination mac address
     pub fn dst_macaddr(&mut self, v: &[u8; 6]) -> &mut Self {
         for i in 0..6 {
-            self.value[i + 5] = v[i];
+            self.value[i + 6] = v[i];
         }
 
         self
@@ -102,9 +110,42 @@ impl EthernetHeader {
     }
 }
 
-/// Ethernet II frame (variable parts of a packet)
+
+/// Parse fields from bytes
+pub fn parse_header_bytes(bytes: &[u8; 14]) -> (MACAddr, MACAddr, EtherType) {
+    use EtherType::*;
+
+    let mut src_macaddr = MACAddr{value: [0_u8; 6]};
+    src_macaddr.value.copy_from_slice(&bytes[0..6]);
+
+    let mut dst_macaddr = MACAddr{value: [0_u8; 6]};
+    dst_macaddr.value.copy_from_slice(&bytes[6..12]);
+
+    let mut ethertype_bytes = [0_u8; 2];
+    ethertype_bytes.copy_from_slice(&bytes[12..14]);
+    let ethertype_int = u16::from_be_bytes(ethertype_bytes);
+    let ethertype = match ethertype_int {
+        x if x == (IPV4 as u16) => IPV4,
+        x if x == (ARP as u16) => ARP,
+        x if x == (VLAN as u16) => VLAN,
+        x if x == (IPV6 as u16) => IPV6,
+        x if x == (EtherCat as u16) => EtherCat,
+        x if x == (PTP as u16) => PTP,
+        _ => Unimplemented
+    };
+
+    return (src_macaddr, dst_macaddr, ethertype)
+}
+
+/// Ethernet II frame containing a UDP packet. 
+/// 
+/// When const generic expressions are more stable, this will be able to generalize
+/// 
+/// to contain any kind of packet that can be reduced to a multiple of 4 bytes.
 ///
-/// P is length of data's byte representation
+/// N is size of IP Options in 32-bit words.
+///
+/// M is size of UDP Data in 32-bit words.
 #[derive(Clone, Debug)]
 pub struct EthernetFrameUDP<const N: usize, const M: usize>
 where
@@ -113,7 +154,7 @@ where
 {
     /// Ethernet frame header
     pub header: EthernetHeader,
-    /// Ethernet payload (likely some kind of IP packet)
+    /// Ethernet payload (only a UDP packet, for now)
     pub payload: UDPPacket<N, M>,
 }
 
@@ -154,13 +195,17 @@ where
             i = i + 1;
         }
 
-        // assert_eq!(i, bytes.len());
-
         bytes
     }
 }
 
 /// Ethernet II packet (including preamble, start-frame delimiter, and interpacket gap)
+/// 
+/// A hardware MAC usually takes the frame as an input and builds a packet from it, so
+/// 
+/// this exercise of building the actual packet from a frame is somewhat academic, but useful for testing
+/// 
+/// and for estimating network utilization.
 #[derive(Clone, Debug)]
 pub struct EthernetPacketUDP<const N: usize, const M: usize>
 where
@@ -195,7 +240,7 @@ where
     /// Calculate ethernet checksum in software
     #[cfg(feature = "crc")]
     pub fn calc_enet_checksum(&self, frame_bytes: &[u8; (4 * N + 20) + (4 * M) + 14 + 8]) -> u32 {
-        let checksum: u32 = crc32fast::hash(&frame_bytes);
+        let checksum: u32 = crc32fast::hash(frame_bytes);
         checksum
     }
 
@@ -212,8 +257,7 @@ where
         let mut bytes = [0_u8; 4 * N + 20 + 4 * M + 14 + 8 + 24];
 
         // Calculate CRC32 checksum over ethernet frame
-        // TODO: this could be done faster using either a persistent Hasher
-        // or a CRC32 peripheral
+        // TODO: this could be done faster using a persistent Hasher
         let frame_bytes = self.frame.to_be_bytes();
         let checksum: u32 = self.calc_enet_checksum(&frame_bytes);
         let checksum_bytes: [u8; 4] = checksum.to_be_bytes();
@@ -240,8 +284,6 @@ where
             i = i + 1;
         }
 
-        // assert_eq!(i, bytes.len());
-
         bytes
     }
 }
@@ -250,6 +292,7 @@ where
 ///
 /// See https://en.wikipedia.org/wiki/EtherType
 #[derive(Clone, Copy, Debug)]
+#[repr(u16)]
 pub enum EtherType {
     /// IPV4
     IPV4 = 0x0800,
@@ -263,4 +306,6 @@ pub enum EtherType {
     EtherCat = 0x88A4,
     /// Precision Time Protocol
     PTP = 0x88A7,
+    /// Catch-all for uncommon types not handled here
+    Unimplemented = 0
 }
